-- Query generated by: templates/unnest_parquet_view.sql.py telemetry_new_profile_parquet_v2 telemetry_derived.telemetry_new_profile_parquet_v2
CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.telemetry_new_profile_parquet_v2` AS
SELECT
  submission_date AS submission,
  * REPLACE (
    (SELECT AS STRUCT environment.* REPLACE ((SELECT AS STRUCT environment.partner.* REPLACE (ARRAY(SELECT * FROM UNNEST(environment.partner.partner_names.list)) AS partner_names)) AS partner)) AS environment
  )
FROM
  `moz-fx-data-shared-prod.telemetry_derived.telemetry_new_profile_parquet_v2`
