CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.experiments`
AS
SELECT
  * EXCEPT (
    scalar_content_webrtc_nicer_stun_retransmits,
    scalar_parent_devtools_aboutdevtools_noinstall_exits,
    scalar_parent_media_blocked_autoplay_no_audio_track_count,
    histogram_content_devtools_computedview_opened_count,
    histogram_content_pwmgr_manage_deleted,
    scalar_parent_devtools_changesview_contextmenu_copy,
    scalar_content_telemetry_process_creation_timestamp_inconsistent,
    scalar_parent_devtools_changesview_contextmenu_copy_rule,
    active_experiment_id,
    plugins_infobar_shown,
    scalar_parent_media_page_had_media_count,
    scalar_parent_webrtc_peerconnection_legacy_callback_stats_used,
    histogram_content_devtools_responsive_opened_count,
    histogram_parent_devtools_webaudioeditor_opened_count,
    histogram_content_pwmgr_manage_copied_password,
    scalar_content_sw_cors_res_for_so_req_count,
    scalar_parent_browser_errors_sample_rate,
    scalar_parent_browser_feeds_livebookmark_item_opened,
    scalar_parent_browser_session_restore_worker_restart_count,
    scalar_parent_formautofill_creditcards_fill_type_autofill_modified,
    histogram_content_devtools_netmonitor_opened_count,
    scalar_parent_encoding_override_used,
    scalar_parent_gfx_hdr_windows_display_colorspace_bitfield,
    scalar_parent_corroborate_system_addons_corrupted,
    scalar_parent_pdf_viewer_print,
    scalar_parent_datasanitization_network_cookie_lifetimepolicy,
    scalar_parent_webrtc_peerconnection_promise_and_callback_stats_used,
    scalar_parent_telemetry_about_telemetry_pageload,
    scalar_parent_devtools_changesview_copy,
    scalar_parent_networking_http_connections_captive_portal,
    plugins_infobar_block,
    scalar_parent_browser_errors_collected_with_stack_count,
    histogram_parent_devtools_developertoolbar_opened_count,
    histogram_content_devtools_picker_eyedropper_opened_count,
    histogram_content_devtools_shadereditor_opened_count,
    histogram_content_devtools_paintflashing_opened_count,
    web_notification_shown,
    scalar_parent_media_autoplay_would_be_allowed_count,
    scalar_parent_navigator_storage_persist_count,
    scalar_parent_media_blocked_no_metadata_endup_no_audio_track,
    scalar_parent_devtools_changesview_contextmenu_copy_declaration,
    scalar_parent_webrtc_peerconnection_datachannel_max_retx_used,
    plugins_infobar_dismissed,
    scalar_parent_aushelper_websense_reg_version,
    scalar_parent_browser_feeds_livebookmark_opened,
    histogram_content_devtools_scratchpad_window_opened_count,
    histogram_content_devtools_animationinspector_opened_count,
    histogram_content_devtools_inspector_opened_count,
    scalar_parent_security_tls_delegated_credentials_txn,
    scalar_parent_datasanitization_privacy_clearonshutdown_downloads,
    crashes_detected_content,
    scalar_content_webrtc_peerconnection_connected,
    scalar_parent_browser_usage_graphite,
    scalar_parent_preferences_read_user_js,
    histogram_content_devtools_developertoolbar_opened_count,
    scalar_parent_datasanitization_privacy_clearonshutdown_history,
    scalar_parent_datasanitization_session_permission_exceptions,
    scalar_parent_telemetry_profile_directory_scans,
    histogram_content_devtools_custom_opened_count,
    histogram_content_devtools_menu_eyedropper_opened_count,
    histogram_content_devtools_jsprofiler_opened_count,
    scalar_content_webrtc_hostnameobfuscation_enabled_failed,
    scalar_parent_browser_session_restore_tabbar_restore_available,
    scalar_parent_devtools_onboarding_is_devtools_user,
    scalar_parent_telemetry_profile_directory_scan_date,
    histogram_content_devtools_layoutview_opened_count,
    scalar_parent_identity_fxaccounts_missed_commands_fetched,
    plugins_infobar_allow,
    scalar_content_media_blocked_autoplay_no_audio_track_count,
    scalar_parent_idb_type_temporary_count,
    scalar_parent_preferences_prevent_accessibility_services,
    histogram_content_devtools_webaudioeditor_opened_count,
    histogram_content_devtools_eyedropper_opened_count,
    histogram_content_pwmgr_manage_copied_username,
    user_pref_marionette_enabled,
    scalar_parent_telemetry_ecosystem_old_send_time,
    scalar_content_webrtc_hostnameobfuscation_disabled_failed,
    scalar_parent_webrtc_hostnameobfuscation_enabled_failed,
    scalar_content_navigator_storage_estimate_count,
    scalar_parent_browser_errors_reported_success_count,
    scalar_parent_storage_sync_api_usage_extensions_using,
    histogram_parent_devtools_canvasdebugger_opened_count,
    scalar_parent_media_allowed_autoplay_no_audio_track_count,
    scalar_parent_telemetry_process_creation_timestamp_inconsistent,
    loop_activity_counter,
    scalar_content_browser_feeds_preview_loaded,
    scalar_parent_media_autoplay_would_not_be_allowed_count,
    scalar_parent_media_blocked_no_metadata,
    histogram_content_devtools_jsdebugger_opened_count,
    scalar_parent_webrtc_peerconnection_datachannel_created,
    scalar_parent_webrtc_hostnameobfuscation_disabled_failed,
    scalar_content_webrtc_hostnameobfuscation_enabled_succeeded,
    scalar_parent_datasanitization_privacy_clearonshutdown_cache,
    scalar_parent_browser_session_restore_tabbar_restore_clicked,
    scalar_gpu_browser_usage_graphite,
    scalar_parent_idb_type_persistent_count,
    scalar_parent_webrtc_nicer_stun_retransmits,
    histogram_content_devtools_styleeditor_opened_count,
    histogram_content_devtools_fontinspector_opened_count,
    scalar_parent_devtools_webreplay_reload_recording,
    scalar_parent_devtools_webreplay_save_recording,
    scalar_content_webrtc_peerconnection_datachannel_max_retx_and_life_used,
    scalar_parent_pdf_viewer_fallback_shown,
    scalar_parent_datasanitization_privacy_sanitize_sanitizeonshutdown,
    scalar_parent_datasanitization_privacy_clearonshutdown_formdata,
    scalar_parent_datasanitization_privacy_clearonshutdown_sessions,
    scalar_parent_services_sync_fxa_verification_method,
    scalar_content_idb_type_persistent_count,
    scalar_content_mediarecorder_recording_count,
    scalar_parent_screenshots_upload,
    scalar_parent_telemetry_os_shutting_down,
    histogram_content_devtools_ruleview_opened_count,
    histogram_content_devtools_dom_opened_count,
    scalar_parent_webrtc_hostnameobfuscation_enabled_succeeded,
    scalar_content_webrtc_peerconnection_promise_stats_used,
    scalar_gpu_telemetry_profile_directory_scans,
    scalar_parent_browser_feeds_livebookmark_count,
    scalar_parent_devtools_webreplay_stop_recording,
    scalar_parent_devtools_webreplay_load_recording,
    scalar_parent_corroborate_omnijar_corrupted,
    scalar_parent_pdf_viewer_used,
    scalar_parent_browser_session_restore_number_of_tabs,
    scalar_content_telemetry_profile_directory_scans,
    scalar_content_webrtc_peerconnection_promise_and_callback_stats_used,
    scalar_parent_mediarecorder_recording_count,
    scalar_parent_sw_cors_res_for_so_req_count,
    scalar_parent_telemetry_pending_operations_highwatermark_reached,
    user_pref_privacy_fuzzyfox_enabled,
    scalar_content_memoryreporter_max_ghost_windows,
    scalar_parent_devtools_aboutdevtools_opened,
    scalar_parent_datasanitization_privacy_clearonshutdown_cookies,
    user_pref_extensions_allow_non_mpc_extensions,
    scalar_content_idb_type_temporary_count,
    scalar_content_webrtc_peerconnection_legacy_callback_stats_used,
    scalar_parent_media_page_had_play_revoked_count,
    scalar_parent_webrtc_peerconnection_promise_stats_used,
    histogram_content_devtools_aboutdebugging_opened_count,
    histogram_content_devtools_webconsole_opened_count,
    histogram_content_devtools_browserconsole_opened_count,
    histogram_content_number_of_profiles,
    user_pref_browser_launcherprocess_enabled,
    scalar_parent_webrtc_peerconnection_datachannel_max_life_used,
    scalar_parent_datasanitization_privacy_clearonshutdown_sitesettings,
    scalar_parent_datasanitization_privacy_clearonshutdown_offlineapps,
    scalar_parent_browser_feeds_preview_loaded,
    scalar_parent_devtools_application_opened_count,
    scalar_parent_dom_contentprocess_troubled_due_to_memory,
    scalar_parent_navigator_storage_estimate_count,
    scalar_parent_telemetry_ecosystem_new_send_time,
    active_experiment_branch,
    histogram_content_devtools_webide_opened_count,
    histogram_content_devtools_scratchpad_opened_count,
    scalar_parent_datasanitization_privacy_clearonshutdown_openwindows,
    scalar_parent_browser_usage_plugin_instantiated,
    scalar_parent_telemetry_persistence_timer_hit_count,
    histogram_content_devtools_memory_opened_count,
    scalar_parent_devtools_changesview_contextmenu,
    scalar_parent_webrtc_peerconnection_datachannel_max_retx_and_life_used,
    scalar_parent_browser_session_restore_number_of_win,
    scalar_content_browser_usage_graphite,
    scalar_parent_preferences_created_new_user_prefs_file,
    scalar_content_webrtc_peerconnection_datachannel_max_retx_used,
    scalar_parent_webrtc_hostnameobfuscation_disabled_succeeded,
    push_api_notification_received,
    plugins_notification_user_action,
    user_prefs,
    histogram_parent_devtools_shadereditor_opened_count,
    scalar_parent_devtools_webreplay_new_recording,
    scalar_content_media_autoplay_default_blocked,
    scalar_parent_networking_data_transferred_captive_portal,
    scalar_parent_browser_feeds_feed_subscribed,
    histogram_content_devtools_storage_opened_count,
    histogram_content_devtools_options_opened_count,
    scalar_content_webrtc_hostnameobfuscation_disabled_succeeded,
    scalar_content_navigator_storage_persist_count,
    scalar_parent_browser_errors_collected_count,
    histogram_content_devtools_canvasdebugger_opened_count,
    histogram_content_devtools_jsbrowserdebugger_opened_count,
    scalar_parent_networking_http_transactions_captive_portal,
    scalar_parent_browser_errors_reported_failure_count,
    scalar_parent_devtools_aboutdevtools_installed,
    scalar_parent_preferences_prefs_file_was_invalid,
    scalar_parent_webrtc_peerconnection_connected
  )
FROM
  `moz-fx-data-shared-prod.telemetry.experiments_v1`
