CREATE OR REPLACE VIEW
  `moz-fx-data-shared-prod.telemetry.mobile_retention_clients`
AS
SELECT
  metric_date,
  submission_date,
  first_seen_date,
  app_name,
  normalized_channel,
  app_version,
  locale,
  country,
  adjust_ad_group,
  adjust_campaign,
  adjust_creative,
  adjust_network,
  play_store_attribution_campaign,
  play_store_attribution_medium,
  play_store_attribution_source,
  meta_attribution_app,
  install_source,
  FALSE AS is_suspicious_device_client,
  lifecycle_stage,
  ping_sent_metric_date,
  ping_sent_week_4,
  active_metric_date,
  retained_week_4,
  new_profile_metric_date,
  repeat_profile,
  retained_week_4_new_profile,
FROM
  `moz-fx-data-shared-prod.fenix.retention_clients`
UNION ALL
SELECT
  metric_date,
  submission_date,
  first_seen_date,
  app_name,
  normalized_channel,
  app_version,
  locale,
  country,
  adjust_ad_group,
  adjust_campaign,
  adjust_creative,
  adjust_network,
  NULL AS play_store_attribution_campaign,
  NULL AS play_store_attribution_medium,
  NULL AS play_store_attribution_source,
  NULL AS meta_attribution_app,
  NULL AS install_source,
  is_suspicious_device_client,
  lifecycle_stage,
  ping_sent_metric_date,
  ping_sent_week_4,
  active_metric_date,
  retained_week_4,
  new_profile_metric_date,
  repeat_profile,
  retained_week_4_new_profile,
FROM
  `moz-fx-data-shared-prod.firefox_ios.retention_clients`
